@@ -14,6 +14,7 @@ from argparse import (
     _VersionAction,
 )
 from functools import total_ordering
+from string import Template
 
 # version detector. Precedence: installed dist, git, 'UNKNOWN'
 try:
@@ -129,13 +130,6 @@ def complete2pattern(opt_complete, shell, choice_type2fn):
     )
 
 
-def replace_format(string, **fmt):
-    """Similar to `string.format(**fmt)` but ignores unknown `{key}`s."""
-    for k, v in fmt.items():
-        string = string.replace("{" + k + "}", v)
-    return string
-
-
 def wordify(string):
     """Replace non-word chars [-. ] with underscores [_]"""
     return string.replace("-", "_").replace(".", " ").replace(" ", "_")
@@ -362,22 +356,22 @@ def complete_bash(parser, root_prefix=None, preamble="", choice_functions=None):
     #   Programmable-Completion.html
     # - https://opensource.com/article/18/3/creating-bash-completion-script
     # - https://stackoverflow.com/questions/12933362
-    return replace_format(
+    return Template(
         """\
 #!/usr/bin/env bash
 # AUTOMATCALLY GENERATED by `shtab`
 
-{subparsers}
+${subparsers}
 
-{option_strings}
+${option_strings}
 
-{compgens}
+${compgens}
 
-{choices}
+${choices}
 
-{nargs}
+${nargs}
 
-{preamble}
+${preamble}
 # $1=COMP_WORDS[1]
 _shtab_compgen_files() {
   compgen -f -- $1  # files
@@ -396,7 +390,7 @@ _shtab_replace_nonword() {
 # set default values (called for the initial parser & any subparsers)
 _set_parser_defaults() {
   local subparsers_var="${prefix}_subparsers[@]"
-  subparsers=${!subparsers_var}
+  sub_parsers=${!subparsers_var}
 
   local current_option_strings_var="${prefix}_option_strings[@]"
   current_option_strings=${!current_option_strings_var}
@@ -437,11 +431,11 @@ _set_new_action() {
 #     hello="world"
 #     x="hello"
 #     ${!x} -> ${hello} -> "world"
-{root_prefix}() {
+${root_prefix}() {
   local completing_word="${COMP_WORDS[COMP_CWORD]}"
   COMPREPLY=()
 
-  prefix={root_prefix}
+  prefix=${root_prefix}
   word_index=0
   _set_parser_defaults
   word_index=1
@@ -451,7 +445,7 @@ _set_new_action() {
   while [ $word_index -ne $COMP_CWORD ]; do
     local this_word="${COMP_WORDS[$word_index]}"
 
-    if [[ -n $subparsers && " ${subparsers[@]} " =~ " ${this_word} " ]]; then
+    if [[ -n $sub_parsers && " ${sub_parsers[@]} " =~ " ${this_word} " ]]; then
       # valid subcommand: add it to the prefix & reset the current action
       prefix="${prefix}_$(_shtab_replace_nonword $this_word)"
       _set_parser_defaults
@@ -491,7 +485,8 @@ _set_new_action() {
   return 0
 }
 
-complete -o filenames -F {root_prefix} {prog}""",
+complete -o filenames -F ${root_prefix} ${prog}"""
+    ).safe_substitute(
         subparsers="\n".join(subparsers),
         option_strings="\n".join(option_strings),
         compgens="\n".join(compgens),
@@ -553,7 +548,7 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
                 else (
                     choice_type2fn[opt.choices[0].type]
                     if isinstance(opt.choices[0], Choice)
-                    else "({})".format(" ".join(opt.choices))
+                    else "({})".format(" ".join(map(str, opt.choices)))
                 )
                 if opt.choices
                 else "",
@@ -570,7 +565,7 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
             else (
                 choice_type2fn[opt.choices[0].type]
                 if isinstance(opt.choices[0], Choice)
-                else "({})".format(" ".join(opt.choices))
+                else "({})".format(" ".join(map(str, opt.choices)))
             )
             if opt.choices
             else "",
@@ -632,37 +627,38 @@ def complete_zsh(parser, root_prefix=None, preamble="", choice_functions=None):
     #   - https://mads-hartmann.com/2017/08/06/
     #     writing-zsh-completion-scripts.html
     #   - http://www.linux-mag.com/id/1106/
-    return replace_format(
+    return Template(
         """\
-#compdef {prog}
+#compdef ${prog}
 
 # AUTOMATCALLY GENERATED by `shtab`
 
-{root_prefix}_options_=(
-  {root_options}
+${root_prefix}_options_=(
+  ${root_options}
 )
 
-{root_prefix}_commands_() {
+${root_prefix}_commands_() {
   local _commands=(
-    {commands}
+    ${commands}
   )
 
-  _describe '{prog} commands' _commands
+  _describe '${prog} commands' _commands
 }
-{subcommands}
-{preamble}
+${subcommands}
+${preamble}
 typeset -A opt_args
 local context state line curcontext="$curcontext"
 
 _arguments \\
-  ${root_prefix}_options_ \\
-  {root_arguments} \\
-  ': :{root_prefix}_commands_' \\
+  $$${root_prefix}_options_ \\
+  ${root_arguments} \\
+  ': :${root_prefix}_commands_' \\
   '*::args:->args'
 
 case $words[1] in
-  {commands_case}
-esac""",
+  ${commands_case}
+esac"""
+    ).safe_substitute(
         root_prefix=root_prefix,
         prog=parser.prog,
         commands="\n    ".join(
@@ -706,11 +702,12 @@ def complete(
     """
     parser  : argparse.ArgumentParser
     shell  : str (bash/zsh)
-    root_prefix  : str, prefix for shell functions to avoid clashes
-      (default: "_{parser.prog}")
-    preamble  : dict, mapping shell to text to prepend to generated script
+    root_prefix  : str
+      prefix for shell functions to avoid clashes (default: "_{parser.prog}")
+    preamble  : dict
+      mapping shell to text to prepend to generated script
       (e.g. `{"bash": "_myprog_custom_function(){ echo hello }"}`)
-    choice_functions  : deprecated.
+    choice_functions  : deprecated
 
     N.B. `parser.add_argument().complete = ...` can be used to define custom
     completions (e.g. filenames). See <../examples/pathcomplete.py>.
@@ -744,10 +741,12 @@ def add_argument_to(
 ):
     """
     parser  : argparse.ArgumentParser
-    option_string  : str or list[str], iff positional (no `-` prefix) then
-      `parser` is assumed to actually be a subparser (subcommand mode)
+    option_string  : str or list[str]
+      iff positional (no `-` prefix) then `parser` is assumed to actually be
+      a subparser (subcommand mode)
     help  : str
-    parent  : argparse.ArgumentParser, required in subcommand mode
+    parent  : argparse.ArgumentParser
+      required in subcommand mode
     """
     if isinstance(
         option_string, str if sys.version_info[0] > 2 else basestring  # NOQA
