@@ -1,11 +1,9 @@
 """
 Tests for `shtab`.
-
-Currently runnable via nosetests, e.g.:
-    shtab$ nose tests -d -v
 """
 import logging
 import subprocess
+import sys
 from argparse import ArgumentParser
 
 import pytest
@@ -20,22 +18,28 @@ class Bash(object):
     def __init__(self, init_script=""):
         self.init = init_script
 
-    def test(self, test="1", failure_message=""):
-        """Equivalent to `bash -c '{init}; [[ {test} ]]'`."""
+    def test(self, cmd="1", failure_message=""):
+        """Equivalent to `bash -c '{init}; [[ {cmd} ]]'`."""
         init = self.init + "\n" if self.init else ""
         proc = subprocess.Popen(
-            ["bash", "-c", "{init}[[ {test} ]]".format(init=init, test=test)]
+            [
+                "bash",
+                "-o",
+                "pipefail",
+                "-ec",
+                "{init}[[ {cmd} ]]".format(init=init, cmd=cmd),
+            ]
         )
         stdout, stderr = proc.communicate()
         assert (
-            0 == proc.wait()
+            0 == proc.wait() and not stdout and not stderr
         ), """\
 {}
 {}
 === stdout ===
 {}=== stderr ===
 {}""".format(
-            failure_message, test, stdout or "", stderr or ""
+            failure_message, cmd, stdout or "", stderr or ""
         )
 
     def compgen(self, compgen_cmd, word, expected_completions, failure_message=""):
@@ -84,7 +88,7 @@ def test_prog_override(shell, caplog, capsys):
     captured = capsys.readouterr()
     assert not captured.err
     if shell == "bash":
-        assert "complete -o nospace -F _shtab_shtab foo" in captured.out
+        assert "complete -o filenames -F _shtab_shtab foo" in captured.out
 
     assert not caplog.record_tuples
 
@@ -98,7 +102,7 @@ def test_prog_scripts(shell, caplog, capsys):
     assert not captured.err
     script_py = [i.strip() for i in captured.out.splitlines() if "script.py" in i]
     if shell == "bash":
-        assert script_py == ["complete -o nospace -F _shtab_shtab script.py"]
+        assert script_py == ["complete -o filenames -F _shtab_shtab script.py"]
     elif shell == "zsh":
         assert script_py == [
             "#compdef script.py",
@@ -114,12 +118,13 @@ def test_prog_scripts(shell, caplog, capsys):
 def test_prefix_override(shell, caplog, capsys):
     with caplog.at_level(logging.INFO):
         main(["-s", shell, "--prefix", "foo", "shtab.main.get_main_parser"])
-
     captured = capsys.readouterr()
+    print(captured.out)
     assert not captured.err
+
     if shell == "bash":
         shell = Bash(captured.out)
-        shell.compgen('-W "$_shtab_foo_options_"', "--h", "--help")
+        shell.compgen('-W "${_shtab_foo_option_strings[*]}"', "--h", "--help")
 
     assert not caplog.record_tuples
 
@@ -133,7 +138,7 @@ def test_complete(shell, caplog):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.compgen('-W "$_shtab_shtab_options_"', "--h", "--help")
+        shell.compgen('-W "${_shtab_shtab_option_strings[*]}"', "--h", "--help")
 
     assert not caplog.record_tuples
 
@@ -148,7 +153,7 @@ def test_positional_choices(shell, caplog):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.compgen('-W "$_shtab_test_commands_"', "o", "one")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "o", "one")
 
     assert not caplog.record_tuples
 
@@ -164,7 +169,7 @@ def test_custom_complete(shell, caplog):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.test('"$($_shtab_test_COMPGEN o)" = "one"')
+        shell.test('"$($_shtab_test_pos_0_COMPGEN o)" = "one"')
 
     assert not caplog.record_tuples
 
@@ -173,7 +178,7 @@ def test_custom_complete(shell, caplog):
 def test_subparser_custom_complete(shell, caplog):
     parser = ArgumentParser(prog="test")
     subparsers = parser.add_subparsers()
-    sub = subparsers.add_parser("sub")
+    sub = subparsers.add_parser("sub", help="help message")
     sub.add_argument("posA").complete = {"bash": "_shtab_test_some_func"}
     preamble = {"bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"}
     with caplog.at_level(logging.INFO):
@@ -182,8 +187,36 @@ def test_subparser_custom_complete(shell, caplog):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.test('"$($_shtab_test_sub_COMPGEN o)" = "one"')
-        shell.test('-z "$($_shtab_test_COMPGEN o)"')
+        shell.compgen('-W "${_shtab_test_subparsers[*]}"', "s", "sub")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "s", "sub")
+        shell.test('"$($_shtab_test_sub_pos_0_COMPGEN o)" = "one"')
+        shell.test('-z "$_shtab_test_COMPGEN"')
+
+    assert not caplog.record_tuples
+
+
+@fix_shell
+@pytest.mark.skipif(sys.version_info[0] == 2, reason="requires Python 3.x")
+def test_subparser_aliases(shell, caplog):
+    parser = ArgumentParser(prog="test")
+    subparsers = parser.add_subparsers()
+    sub = subparsers.add_parser("sub", aliases=["xsub", "ysub"], help="help message")
+    sub.add_argument("posA").complete = {"bash": "_shtab_test_some_func"}
+    preamble = {"bash": "_shtab_test_some_func() { compgen -W 'one two' -- $1 ;}"}
+    with caplog.at_level(logging.INFO):
+        completion = shtab.complete(parser, shell=shell, preamble=preamble)
+    print(completion)
+
+    if shell == "bash":
+        shell = Bash(completion)
+        shell.compgen('-W "${_shtab_test_subparsers[*]}"', "s", "sub")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "s", "sub")
+        shell.compgen('-W "${_shtab_test_subparsers[*]}"', "x", "xsub")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "x", "xsub")
+        shell.compgen('-W "${_shtab_test_subparsers[*]}"', "y", "ysub")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "y", "ysub")
+        shell.test('"$($_shtab_test_sub_pos_0_COMPGEN o)" = "one"')
+        shell.test('-z "$_shtab_test_COMPGEN"')
 
     assert not caplog.record_tuples
 
@@ -198,7 +231,7 @@ def test_add_argument_to_optional(shell, caplog):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.compgen('-W "$_shtab_test_options_"', "--s", "--shell")
+        shell.compgen('-W "${_shtab_test_option_strings[*]}"', "--s", "--shell")
 
     assert not caplog.record_tuples
 
@@ -207,7 +240,7 @@ def test_add_argument_to_optional(shell, caplog):
 def test_add_argument_to_positional(shell, caplog, capsys):
     parser = ArgumentParser(prog="test")
     subparsers = parser.add_subparsers()
-    sub = subparsers.add_parser("completion")
+    sub = subparsers.add_parser("completion", help="help message")
     shtab.add_argument_to(sub, "shell", parent=parser)
     from argparse import Namespace
 
@@ -224,9 +257,10 @@ def test_add_argument_to_positional(shell, caplog, capsys):
 
     if shell == "bash":
         shell = Bash(completion)
-        shell.compgen('-W "$_shtab_test_commands_"', "c", "completion")
-        shell.compgen('-W "$_shtab_test_completion"', "ba", "bash")
-        shell.compgen('-W "$_shtab_test_completion"', "z", "zsh")
+        shell.compgen('-W "${_shtab_test_subparsers[*]}"', "c", "completion")
+        shell.compgen('-W "$_shtab_test_pos_0_choices"', "c", "completion")
+        shell.compgen('-W "$_shtab_test_completion_pos_0_choices"', "ba", "bash")
+        shell.compgen('-W "$_shtab_test_completion_pos_0_choices"', "z", "zsh")
 
     assert not caplog.record_tuples
 
